@@ -26,7 +26,7 @@
 
 #include <QDBusMetaType>
 
-OpenswanWidget::OpenswanWidget(const NetworkManager::VpnSetting::Ptr &setting, QWidget* parent, Qt::WindowFlags f)
+OpenswanWidget::OpenswanWidget(const NetworkManager::VpnSetting::Ptr &setting, QWidget *parent, Qt::WindowFlags f)
     : SettingWidget(setting, parent, f)
     , m_ui(new Ui::OpenswanWidget)
     , m_setting(setting)
@@ -35,17 +35,15 @@ OpenswanWidget::OpenswanWidget(const NetworkManager::VpnSetting::Ptr &setting, Q
 
     m_ui->setupUi(this);
 
-    connect(m_ui->cbUsernamePasswordMode, static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged), this, &OpenswanWidget::userPasswordTypeChanged);
-    connect(m_ui->cbGroupPasswordMode, static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged), this, &OpenswanWidget::groupPasswordTypeChanged);
-
-    connect(m_ui->ckShowPasswords, &QCheckBox::toggled, this, &OpenswanWidget::showPassword);
+    m_ui->groupPassword->setPasswordOptionsEnabled(true);
+    m_ui->userPassword->setPasswordOptionsEnabled(true);
 
     connect(m_ui->gateway, &QLineEdit::textChanged, this, &OpenswanWidget::slotWidgetChanged);
     connect(m_ui->groupname, &QLineEdit::textChanged, this, &OpenswanWidget::slotWidgetChanged);
 
     KAcceleratorManager::manage(this);
 
-    if (m_setting) {
+    if (setting && !setting->isNull()) {
         loadConfig(setting);
     }
 }
@@ -71,22 +69,22 @@ void OpenswanWidget::loadConfig(const NetworkManager::Setting::Ptr &setting)
         m_ui->groupname->setText(groupName);
     }
 
-    const QString userPasswordMode = data.value(NM_OPENSWAN_XAUTH_PASSWORD_INPUT_MODES);
-    if (userPasswordMode == NM_OPENSWAN_PW_TYPE_SAVE) {
-        m_ui->cbUsernamePasswordMode->setCurrentIndex(0);
-    } else if (userPasswordMode == NM_OPENSWAN_PW_TYPE_ASK) {
-        m_ui->cbUsernamePasswordMode->setCurrentIndex(1);
-    } else if (userPasswordMode == NM_OPENSWAN_PW_TYPE_UNUSED) {
-        m_ui->cbUsernamePasswordMode->setCurrentIndex(2);
+    const NetworkManager::Setting::SecretFlags groupPasswordFlag = static_cast<NetworkManager::Setting::SecretFlags>(data.value(NM_OPENSWAN_PSK_VALUE"-flags").toInt());
+    if (groupPasswordFlag == NetworkManager::Setting::None) {
+        m_ui->groupPassword->setPasswordOption(PasswordField::StoreForAllUsers);
+    } else if (groupPasswordFlag == NetworkManager::Setting::AgentOwned) {
+        m_ui->groupPassword->setPasswordOption(PasswordField::StoreForUser);
+    } else {
+        m_ui->groupPassword->setPasswordOption(PasswordField::AlwaysAsk);
     }
 
-    const QString groupPasswordMode = data.value(NM_OPENSWAN_PSK_INPUT_MODES);
-    if (groupPasswordMode == NM_OPENSWAN_PW_TYPE_SAVE) {
-        m_ui->cbGroupPasswordMode->setCurrentIndex(0);
-    } else if (groupPasswordMode == NM_OPENSWAN_PW_TYPE_ASK) {
-        m_ui->cbGroupPasswordMode->setCurrentIndex(1);
-    } else if (groupPasswordMode == NM_OPENSWAN_PW_TYPE_UNUSED) {
-        m_ui->cbGroupPasswordMode->setCurrentIndex(2);
+    const NetworkManager::Setting::SecretFlags userPasswordFlag = static_cast<NetworkManager::Setting::SecretFlags>(data.value(NM_OPENSWAN_XAUTH_PASSWORD"-flags").toInt());
+    if (userPasswordFlag == NetworkManager::Setting::None) {
+        m_ui->userPassword->setPasswordOption(PasswordField::StoreForAllUsers);
+    } else if (userPasswordFlag == NetworkManager::Setting::AgentOwned) {
+        m_ui->userPassword->setPasswordOption(PasswordField::StoreForUser);
+    } else {
+        m_ui->userPassword->setPasswordOption(PasswordField::AlwaysAsk);
     }
 
     const QString username = data.value(NM_OPENSWAN_LEFTXAUTHUSER);
@@ -131,7 +129,7 @@ void OpenswanWidget::loadSecrets(const NetworkManager::Setting::Ptr &setting)
     }
 }
 
-QVariantMap OpenswanWidget::setting(bool agentOwned) const
+QVariantMap OpenswanWidget::setting() const
 {
     NetworkManager::VpnSetting setting;
     setting.setServiceType(QLatin1String(NM_DBUS_SERVICE_OPENSWAN));
@@ -150,40 +148,28 @@ QVariantMap OpenswanWidget::setting(bool agentOwned) const
         secrets.insert(NM_OPENSWAN_XAUTH_PASSWORD, m_ui->userPassword->text());
     }
 
-    const int usernamePasswordMode = m_ui->cbUsernamePasswordMode->currentIndex();
-    if (usernamePasswordMode == 0) {
+    if (m_ui->userPassword->passwordOption() == PasswordField::StoreForAllUsers) {
         data.insert(NM_OPENSWAN_XAUTH_PASSWORD_INPUT_MODES, NM_OPENSWAN_PW_TYPE_SAVE);
-        if (agentOwned) {
-            data.insert(NM_OPENSWAN_XAUTH_PASSWORD"-flags", QString::number(NetworkManager::Setting::AgentOwned));
-        } else {
-            data.insert(NM_OPENSWAN_XAUTH_PASSWORD"-flags", QString::number(NetworkManager::Setting::None));
-        }
-    } else if (usernamePasswordMode == 1) {
+        data.insert(NM_OPENSWAN_XAUTH_PASSWORD"-flags", QString::number(NetworkManager::Setting::None));
+    } else if (m_ui->userPassword->passwordOption() == PasswordField::StoreForUser) {
+        data.insert(NM_OPENSWAN_XAUTH_PASSWORD"-flags", QString::number(NetworkManager::Setting::AgentOwned));
+    } else {
         data.insert(NM_OPENSWAN_XAUTH_PASSWORD_INPUT_MODES, NM_OPENSWAN_PW_TYPE_ASK);
         data.insert(NM_OPENSWAN_XAUTH_PASSWORD"-flags", QString::number(NetworkManager::Setting::NotSaved));
-    } else {
-        data.insert(NM_OPENSWAN_XAUTH_PASSWORD_INPUT_MODES, NM_OPENSWAN_PW_TYPE_UNUSED);
-        data.insert(NM_OPENSWAN_XAUTH_PASSWORD"-flags", QString::number(NetworkManager::Setting::NotRequired));
     }
 
     if (!m_ui->groupPassword->text().isEmpty()) {
         secrets.insert(NM_OPENSWAN_PSK_VALUE, m_ui->groupPassword->text());
     }
 
-    const int groupPasswordMode = m_ui->cbGroupPasswordMode->currentIndex();
-    if (groupPasswordMode == 0) {
+    if (m_ui->groupPassword->passwordOption() == PasswordField::StoreForAllUsers) {
         data.insert(NM_OPENSWAN_PSK_INPUT_MODES, NM_OPENSWAN_PW_TYPE_SAVE);
-        if (agentOwned) {
-            data.insert(NM_OPENSWAN_PSK_VALUE"-flags", QString::number(NetworkManager::Setting::AgentOwned));
-        } else {
-            data.insert(NM_OPENSWAN_PSK_VALUE"-flags", QString::number(NetworkManager::Setting::None));
-        }
-    } else if (groupPasswordMode == 1) {
+        data.insert(NM_OPENSWAN_PSK_VALUE"-flags", QString::number(NetworkManager::Setting::None));
+    } else if (m_ui->groupPassword->passwordOption() == PasswordField::StoreForUser) {
+        data.insert(NM_OPENSWAN_PSK_VALUE"-flags", QString::number(NetworkManager::Setting::AgentOwned));
+    } else {
         data.insert(NM_OPENSWAN_PSK_INPUT_MODES, NM_OPENSWAN_PW_TYPE_ASK);
         data.insert(NM_OPENSWAN_PSK_VALUE"-flags", QString::number(NetworkManager::Setting::NotSaved));
-    } else {
-        data.insert(NM_OPENSWAN_PSK_INPUT_MODES, NM_OPENSWAN_PW_TYPE_UNUSED);
-        data.insert(NM_OPENSWAN_PSK_VALUE"-flags", QString::number(NetworkManager::Setting::NotRequired));
     }
 
     if (!m_ui->username->text().isEmpty()) {
@@ -207,35 +193,6 @@ QVariantMap OpenswanWidget::setting(bool agentOwned) const
     return setting.toMap();
 }
 
-void OpenswanWidget::userPasswordTypeChanged(int index)
-{
-    if (index == 1 || index == 2) {
-        m_ui->userPassword->setEnabled(false);
-    } else {
-        m_ui->userPassword->setEnabled(true);
-    }
-}
-
-void OpenswanWidget::groupPasswordTypeChanged(int index)
-{
-    if (index == 1 || index == 2) {
-        m_ui->groupPassword->setEnabled(false);
-    } else {
-        m_ui->groupPassword->setEnabled(true);
-    }
-}
-
-void OpenswanWidget::showPassword(bool show)
-{
-    if (show) {
-        m_ui->userPassword->setEchoMode(QLineEdit::Normal);
-        m_ui->groupPassword->setEchoMode(QLineEdit::Normal);
-    } else {
-        m_ui->userPassword->setEchoMode(QLineEdit::Password);
-        m_ui->groupPassword->setEchoMode(QLineEdit::Password);
-    }
-}
-
 bool OpenswanWidget::isValid() const
 {
     return !m_ui->gateway->text().isEmpty() && !m_ui->groupname->text().isEmpty();
